@@ -23,6 +23,8 @@ class OptionsManager(object):
         :param path: ini文件的路径，默认读取模块文件夹下的
         """
         self.ini_path = str(Path(__file__).parent / 'configs.ini') if path == 'default' or path is None else path
+        if not Path(self.ini_path).exists():
+            raise FileNotFoundError('ini文件不存在。')
         self._conf = RawConfigParser()
         self._conf.read(self.ini_path, encoding='utf-8')
 
@@ -292,7 +294,7 @@ class SessionOptions(object):
         :param proxies: 参数值
         :return: None
         """
-        self._proxies = proxies
+        self.set_proxies(proxies)
 
     @hooks.setter
     def hooks(self, hooks: dict) -> None:
@@ -392,6 +394,16 @@ class SessionOptions(object):
 
         return self
 
+    def set_proxies(self, proxies: dict):
+        """设置proxies参数           \n
+        {'http': 'http://xx.xx.xx.xx:xxxx',
+         'https': 'http://xx.xx.xx.xx:xxxx'}
+        :param proxies: 参数值
+        :return: None
+        """
+        self._proxies = proxies
+        return self
+
     def save(self, path: str = None):
         """保存设置到文件                                              \n
         :param path: ini文件的路径，传入 'default' 保存到默认ini文件
@@ -600,27 +612,23 @@ class DriverOptions(Options):
         :param cache_path: 缓存路径
         :return: 当前对象
         """
-
-        def format_path(path: str) -> str:
-            return path.replace('/', '\\')
-
         if driver_path is not None:
-            self._driver_path = format_path(driver_path)
+            self._driver_path = driver_path
 
         if chrome_path is not None:
-            self.binary_location = format_path(chrome_path)
+            self.binary_location = chrome_path
 
         if debugger_address is not None:
             self.debugger_address = debugger_address
 
         if download_path is not None:
-            self.experimental_options['prefs']['download.default_directory'] = format_path(download_path)
+            self.experimental_options['prefs']['download.default_directory'] = download_path
 
         if user_data_path is not None:
-            self.set_argument('--user-data-dir', format_path(user_data_path))
+            self.set_argument('--user-data-dir', user_data_path)
 
         if cache_path is not None:
-            self.set_argument('--disk-cache-dir', format_path(cache_path))
+            self.set_argument('--disk-cache-dir', cache_path)
 
         return self
 
@@ -647,7 +655,7 @@ def _dict_to_chrome_options(options: dict) -> Options:
         # 启动参数
         if options.get('arguments', None):
             if not isinstance(options['arguments'], list):
-                raise Exception(f"Arguments need list，not {type(options['arguments'])}.")
+                raise Exception(f"参数必须为list，现在是：{type(options['arguments'])}。")
 
             for arg in options['arguments']:
                 chrome_options.add_argument(arg)
@@ -655,7 +663,7 @@ def _dict_to_chrome_options(options: dict) -> Options:
         # 加载插件
         if options.get('extension_files', None):
             if not isinstance(options['extension_files'], list):
-                raise Exception(f'Extension files need list，not {type(options["extension_files"])}.')
+                raise Exception(f'extension_files必须是list，现在是：{type(options["extension_files"])}。')
 
             for arg in options['extension_files']:
                 chrome_options.add_extension(arg)
@@ -663,7 +671,7 @@ def _dict_to_chrome_options(options: dict) -> Options:
         # 扩展设置
         if options.get('extensions', None):
             if not isinstance(options['extensions'], list):
-                raise Exception(f'Extensions need list，not {type(options["extensions"])}.')
+                raise Exception(f'extensions必须是list，现在是：{type(options["extensions"])}。')
 
             for arg in options['extensions']:
                 chrome_options.add_encoded_extension(arg)
@@ -671,7 +679,7 @@ def _dict_to_chrome_options(options: dict) -> Options:
         # 实验性质的设置参数
         if options.get('experimental_options', None):
             if not isinstance(options['experimental_options'], dict):
-                raise Exception(f'Experimental options need dict，not {type(options["experimental_options"])}.')
+                raise Exception(f'experimental_options必须是dict，现在是：{type(options["experimental_options"])}。')
 
             for i in options['experimental_options']:
                 chrome_options.add_experimental_option(i, options['experimental_options'][i])
@@ -753,7 +761,6 @@ def _cookie_to_dict(cookie: Union[Cookie, str, dict]) -> dict:
             attr_val = attr.lstrip().split('=')
 
             if key == 0:
-                # TODO: 检查
                 cookie_dict['name'] = attr_val[0]
                 cookie_dict['value'] = attr_val[1] if len(attr_val) == 2 else ''
             else:
@@ -762,7 +769,7 @@ def _cookie_to_dict(cookie: Union[Cookie, str, dict]) -> dict:
         return cookie_dict
 
     else:
-        raise TypeError
+        raise TypeError('cookie参数必须为Cookie、str或dict类型。')
 
     return cookie_dict
 
@@ -782,6 +789,6 @@ def _cookies_to_tuple(cookies: Union[RequestsCookieJar, list, tuple, str, dict])
         cookies = tuple({'name': cookie, 'value': cookies[cookie]} for cookie in cookies)
 
     else:
-        raise TypeError
+        raise TypeError('cookies参数必须为RequestsCookieJar、list、tuple、str或dict类型。')
 
     return cookies
