@@ -6,8 +6,7 @@
 """
 from os import sep
 from pathlib import Path
-from re import sub
-from time import time
+from time import time, perf_counter
 from typing import Union, List, Any, Tuple
 
 from selenium.common.exceptions import TimeoutException, JavascriptException, InvalidElementStateException
@@ -17,7 +16,8 @@ from selenium.webdriver.support import expected_conditions as ec
 from selenium.webdriver.support.wait import WebDriverWait
 
 from .base import DrissionElement, BaseElement
-from .common import str_to_loc, get_available_file_name, translate_loc, format_html
+from .common import str_to_loc, get_usable_path, translate_loc, format_html, get_ele_txt
+from .session_element import make_session_ele
 
 
 class DriverElement(DrissionElement):
@@ -33,16 +33,14 @@ class DriverElement(DrissionElement):
 
     def __call__(self,
                  loc_or_str: Union[Tuple[str, str], str],
-                 mode: str = 'single',
                  timeout: float = None):
-        """在内部查找元素                                            \n
+        """在内部查找元素                                             \n
         例：ele2 = ele1('@id=ele_id')                               \n
         :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
-        :param mode: 'single' 或 'all'，对应查找一个或全部
         :param timeout: 超时时间
-        :return: DriverElement对象
+        :return: DriverElement对象或属性、文本
         """
-        return super().__call__(loc_or_str, mode, timeout)
+        return self.ele(loc_or_str, timeout)
 
     # -----------------共有属性和方法-------------------
     @property
@@ -53,12 +51,12 @@ class DriverElement(DrissionElement):
     @property
     def html(self) -> str:
         """返回元素outerHTML文本"""
-        return self.attr('outerHTML')
+        return self.inner_ele.get_attribute('outerHTML')
 
     @property
     def inner_html(self) -> str:
         """返回元素innerHTML文本"""
-        return self.attr('innerHTML')
+        return self.inner_ele.get_attribute('innerHTML')
 
     @property
     def attrs(self) -> dict:
@@ -82,18 +80,18 @@ class DriverElement(DrissionElement):
     @property
     def text(self) -> str:
         """返回元素内所有文本"""
-        # return format_html(self.inner_ele.get_attribute('innerText'), False)
-        re_str = self.inner_ele.get_attribute('innerText')
-        re_str = sub(r'\n{2,}', '\n', re_str)
-        re_str = sub(r' {2,}', ' ', re_str)
-
-        return format_html(re_str.strip('\n '), False)
+        return get_ele_txt(make_session_ele(self.html))
 
     @property
     def raw_text(self) -> str:
         """返回未格式化处理的元素内文本"""
         return self.inner_ele.get_attribute('innerText')
 
+    @property
+    def parent(self):
+        """返回父级元素"""
+        return self.parents()
+
     def parents(self, num: int = 1):
         """返回上面第num级父元素              \n
         :param num: 第几级父元素
@@ -107,43 +105,26 @@ class DriverElement(DrissionElement):
         :param attr: 属性名
         :return: 属性值文本
         """
-        # attr = 'innerText' if attr == 'text' else attr
-        if attr in ('text', 'innerText'):
+        if attr == 'text':
             return self.text
-
-        return format_html(self.inner_ele.get_attribute(attr))
+        elif attr == 'innerText':
+            return self.raw_text
+        elif attr in ('html', 'outerHTML'):
+            return self.html
+        elif attr == 'innerHTML':
+            return self.inner_html
+        else:
+            return format_html(self.inner_ele.get_attribute(attr))
 
     def ele(self,
             loc_or_str: Union[Tuple[str, str], str],
-            mode: str = None,
             timeout: float = None):
-        """返回当前元素下级符合条件的子元素、属性或节点文本，默认返回第一个                                      \n
+        """返回当前元素下级符合条件的第一个元素、属性或节点文本                                      \n
         :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
-        :param mode: 'single' 或 'all'，对应查找一个或全部
         :param timeout: 查找元素超时时间
-        :return: DriverElement对象
+        :return: DriverElement对象或属性、文本
         """
-        if isinstance(loc_or_str, (str, tuple)):
-            if isinstance(loc_or_str, str):
-                loc_or_str = str_to_loc(loc_or_str)
-            else:
-                if len(loc_or_str) != 2:
-                    raise ValueError("Len of loc_or_str must be 2 when it's a tuple.")
-                loc_or_str = translate_loc(loc_or_str)
-        else:
-            raise ValueError('Argument loc_or_str can only be tuple or str.')
-
-        loc_str = loc_or_str[1]
-
-        if loc_or_str[0] == 'xpath' and loc_or_str[1].lstrip().startswith('/'):
-            loc_str = f'.{loc_str}'
-
-        if loc_or_str[0] == 'css selector' and loc_or_str[1].lstrip().startswith('>'):
-            loc_str = f'{self.css_path}{loc_or_str[1]}'
-
-        loc_or_str = loc_or_str[0], loc_str
-
-        return execute_driver_find(self, loc_or_str, mode, timeout)
+        return self._ele(loc_or_str, timeout)
 
     def eles(self,
              loc_or_str: Union[Tuple[str, str], str],
@@ -151,9 +132,35 @@ class DriverElement(DrissionElement):
         """返回当前元素下级所有符合条件的子元素、属性或节点文本                                                   \n
         :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
         :param timeout: 查找元素超时时间
-        :return: DriverElement对象组成的列表
+        :return: DriverElement对象或属性、文本组成的列表
         """
-        return self.ele(loc_or_str, mode='all', timeout=timeout)
+        return self._ele(loc_or_str, timeout=timeout, single=False)
+
+    def s_ele(self, loc_or_ele=None):
+        """查找第一个符合条件的元素以SessionElement形式返回，处理复杂页面时效率很高        \n
+        :param loc_or_ele: 元素的定位信息，可以是loc元组，或查询字符串
+        :return: SessionElement对象或属性、文本
+        """
+        return make_session_ele(self, loc_or_ele)
+
+    def s_eles(self, loc_or_str: Union[Tuple[str, str], str]):
+        """查找所有符合条件的元素以SessionElement列表形式返回                         \n
+        :param loc_or_str: 定位符
+        :return: SessionElement或属性、文本组成的列表
+        """
+        return make_session_ele(self, loc_or_str, single=False)
+
+    def _ele(self,
+             loc_or_str: Union[Tuple[str, str], str],
+             timeout: float = None,
+             single: bool = True):
+        """返回当前元素下级符合条件的子元素、属性或节点文本，默认返回第一个                                      \n
+        :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
+        :param timeout: 查找元素超时时间
+        :param single: True则返回第一个，False则返回全部
+        :return: DriverElement对象
+        """
+        return make_driver_ele(self, loc_or_str, single, timeout)
 
     def _get_ele_path(self, mode) -> str:
         """返获取css路径或xpath路径"""
@@ -174,7 +181,7 @@ class DriverElement(DrissionElement):
             txt5 = '''return path.substr(1);'''
 
         else:
-            raise ValueError(f"Argument mode can only be 'xpath' or 'css', not '{mode}'.")
+            raise ValueError(f"mode参数只能是'xpath'或'css'，现在是：'{mode}'。")
 
         js = '''
         function e(el) {
@@ -274,7 +281,6 @@ class DriverElement(DrissionElement):
         """
         if not by_js:
             timeout = timeout if timeout is not None else self.page.timeout
-            from time import perf_counter
             t1 = perf_counter()
             while perf_counter() - t1 <= timeout:
                 try:
@@ -332,37 +338,34 @@ class DriverElement(DrissionElement):
         from selenium.webdriver import ActionChains
         ActionChains(self.page.driver).move_to_element_with_offset(self.inner_ele, x, y).context_click().perform()
 
-    def input(self, vals: Union[str, tuple], clear: bool = True) -> None:
-        """输入文本或组合键，可用于所有场合          \n
+    def input(self, vals: Union[str, tuple], clear: bool = True, insure_input: bool = True) -> None:
+        """输入文本或组合键，也可用于输入文件路径到input元素（文件间用\n间隔）                  \n
         :param vals: 文本值或按键组合
         :param clear: 输入前是否清空文本框
+        :param insure_input: 确保输入正确，解决文本框有时输入失效的问题，不能用于输入组合键
         :return: None
         """
-        if clear:
-            self.clear()
-
-        self.inner_ele.send_keys(*vals)
-
-    def input_txt(self, txt: Union[str, tuple], clear: bool = True) -> None:
-        """专门用于输入文本框，解决文本框有时输入失效的问题       \n
-        :param txt: 文本值
-        :param clear: 输入前是否清空文本框
-        :return: None
-        """
-        enter = '\n' if txt.endswith('\n') else None
-        full_txt = txt if clear else f'{self.attr("value")}{txt}'
-        full_txt = full_txt.rstrip('\n')
-        from time import perf_counter
-
-        t1 = perf_counter()
-        while self.attr('value') != full_txt and perf_counter() - t1 > self.page.timeout:
+        if not insure_input:  # 普通输入
             if clear:
-                self.clear()
+                self.inner_ele.clear()
 
-            self.inner_ele.send_keys(txt)
+            self.inner_ele.send_keys(*vals)
 
-        if enter:
-            self.inner_ele.send_keys(enter)
+        else:  # 确保输入正确
+            if not isinstance(vals, str):
+                raise TypeError('insure_input参数生效时vals只能接收str数据。')
+            enter = '\n' if vals.endswith('\n') else None
+            full_txt = vals if clear else f'{self.attr("value")}{vals}'
+            full_txt = full_txt.rstrip('\n')
+
+            t1 = perf_counter()
+            while self.is_valid() and self.attr('value') != full_txt and perf_counter() - t1 <= self.page.timeout:
+                if clear:
+                    self.inner_ele.clear()
+                self.inner_ele.send_keys(vals)
+
+            if enter:
+                self.inner_ele.send_keys(enter)
 
     def run_script(self, script: str, *args) -> Any:
         """执行js代码，传入自己为第一个参数  \n
@@ -378,7 +381,7 @@ class DriverElement(DrissionElement):
 
     def clear(self) -> None:
         """清空元素文本"""
-        self.inner_ele.clear()
+        self.input('')
 
     def is_selected(self) -> bool:
         """是否选中"""
@@ -409,8 +412,8 @@ class DriverElement(DrissionElement):
         name = filename or self.tag
         path = Path(path).absolute()
         path.mkdir(parents=True, exist_ok=True)
-        name = f'{name}.png' if not name.endswith('.png') else name
-        name = get_available_file_name(str(path), name)
+        if not name.lower().endswith('.png'):
+            name = f'{name}.png'
 
         # 等待元素加载完成
         if self.tag == 'img':
@@ -419,7 +422,7 @@ class DriverElement(DrissionElement):
             while not self.run_script(js):
                 pass
 
-        img_path = f'{path}{sep}{name}'
+        img_path = str(get_usable_path(f'{path}{sep}{name}'))
         self.inner_ele.screenshot(img_path)
 
         return img_path
@@ -465,7 +468,7 @@ class DriverElement(DrissionElement):
             self.run_script(f'arguments[0].removeAttribute("{attr}");')
             return True
         except:
-            raise False
+            return False
 
     def drag(self, x: int, y: int, speed: int = 40, shake: bool = True) -> bool:
         """拖拽当前元素到相对位置                   \n
@@ -496,7 +499,7 @@ class DriverElement(DrissionElement):
         elif isinstance(ele_or_loc, tuple):
             target_x, target_y = ele_or_loc
         else:
-            raise TypeError('Need DriverElement, WebElement object or coordinate information.')
+            raise TypeError('需要DriverElement、WebElement对象或坐标。')
 
         current_x = self.location['x'] + self.size['width'] // 2
         current_y = self.location['y'] + self.size['height'] // 2
@@ -531,69 +534,83 @@ class DriverElement(DrissionElement):
         ActionChains(self.page.driver).move_to_element(self.inner_ele).perform()
 
 
-def execute_driver_find(page_or_ele,
-                        loc: Tuple[str, str],
-                        mode: str = 'single',
-                        timeout: float = None) -> Union[DriverElement, List[DriverElement], str, None]:
+def make_driver_ele(page_or_ele,
+                    loc: Union[str, Tuple[str, str]],
+                    single: bool = True,
+                    timeout: float = None) -> Union[DriverElement, List[DriverElement], str, None]:
     """执行driver模式元素的查找                               \n
     页面查找元素及元素查找下级元素皆使用此方法                   \n
     :param page_or_ele: DriverPage对象或DriverElement对象
     :param loc: 元素定位元组
-    :param mode: 'single' 或 'all'，对应获取第一个或全部
+    :param single: True则返回第一个，False则返回全部
     :param timeout: 查找元素超时时间
     :return: 返回DriverElement元素或它们组成的列表
     """
-    mode = mode or 'single'
-    if mode not in ('single', 'all'):
-        raise ValueError(f"Argument mode can only be 'single' or 'all', not '{mode}'.")
+    # ---------------处理定位符---------------
+    if isinstance(loc, str):
+        loc = str_to_loc(loc)
+    elif isinstance(loc, tuple):
+        loc = translate_loc(loc)
+    else:
+        raise ValueError("定位符必须为str或长度为2的tuple。")
+
+    # ---------------设置 page 和 driver---------------
+    if isinstance(page_or_ele, BaseElement):  # 传入DriverElement 或 ShadowRootElement
+        loc_str = loc[1]
+        if loc[0] == 'xpath' and loc[1].lstrip().startswith('/'):
+            loc_str = f'.{loc_str}'
+        elif loc[0] == 'css selector' and loc[1].lstrip().startswith('>') and isinstance(page_or_ele, DriverElement):
+            loc_str = f'{page_or_ele.css_path}{loc[1]}'
+        loc = loc[0], loc_str
 
-    if isinstance(page_or_ele, BaseElement):
         page = page_or_ele.page
         driver = page_or_ele.inner_ele
+
     else:  # 传入的是DriverPage对象
         page = page_or_ele
         driver = page_or_ele.driver
 
-    # 设置等待对象
+    # -----------------设置等待对象-----------------
     if timeout is not None and timeout != page.timeout:
         wait = WebDriverWait(driver, timeout=timeout)
     else:
         page.wait_object._driver = driver
         wait = page.wait_object
 
+    # ---------------执行查找-----------------
     try:
         # 使用xpath查找
         if loc[0] == 'xpath':
-            return wait.until(ElementsByXpath(page, loc[1], mode, timeout))
+            return wait.until(ElementsByXpath(page, loc[1], single, timeout))
 
         # 使用css selector查找
         else:
-            if mode == 'single':
+            if single:
                 return DriverElement(wait.until(ec.presence_of_element_located(loc)), page)
-            elif mode == 'all':
+            else:
                 eles = wait.until(ec.presence_of_all_elements_located(loc))
                 return [DriverElement(ele, page) for ele in eles]
 
     except TimeoutException:
-        return [] if mode == 'all' else None
+        return [] if not single else None
 
     except InvalidElementStateException:
-        raise ValueError(f'Invalid query syntax. {loc}')
+        raise ValueError(f'无效的查找语句：{loc}')
 
 
 class ElementsByXpath(object):
     """用js通过xpath获取元素、节点或属性，与WebDriverWait配合使用"""
 
-    def __init__(self, page, xpath: str = None, mode: str = 'all', timeout: float = 10):
+    def __init__(self, page, xpath: str = None, single: bool = False, timeout: float = 10):
         """
         :param page: DrissionPage对象
         :param xpath: xpath文本
-        :param mode: 'all' 或 'single'
+        :param single: True则返回第一个，False则返回全部
         :param timeout: 超时时间
         """
         self.page = page
         self.xpath = xpath
-        self.mode = mode
+        self.single = single
         self.timeout = timeout
 
     def __call__(self, ele_or_driver: Union[WebDriver, WebElement]) \
@@ -651,7 +668,7 @@ class ElementsByXpath(object):
             driver, the_node = ele_or_driver.parent, ele_or_driver
 
         # 把lxml元素对象包装成DriverElement对象并按需要返回第一个或全部
-        if self.mode == 'single':
+        if self.single:
             try:
                 e = get_nodes(the_node, xpath_txt=self.xpath, type_txt='9')
 
@@ -672,7 +689,7 @@ class ElementsByXpath(object):
                 else:
                     return None
 
-        elif self.mode == 'all':
+        else:  # 返回全部
             return ([DriverElement(x, self.page) if isinstance(x, WebElement)
                      else format_html(x)
                      for x in get_nodes(the_node, xpath_txt=self.xpath)
@@ -687,11 +704,11 @@ class Select(object):
         :param ele: select 元素对象
         """
         if ele.tag != 'select':
-            raise TypeError(f"Select only works on <select> elements, not on {ele.tag}")
+            raise TypeError(f"select方法只能在<select>元素使用，现在是：{ele.tag}。")
 
-        from selenium.webdriver.support.select import Select as sl
+        from selenium.webdriver.support.select import Select
         self.inner_ele = ele
-        self.select_ele = sl(ele.inner_ele)
+        self.select_ele = Select(ele.inner_ele)
 
     def __call__(self,
                  text_value_index: Union[str, int, list, tuple] = None,
@@ -827,7 +844,7 @@ class Select(object):
     def invert(self) -> None:
         """反选"""
         if not self.is_multi:
-            raise NotImplementedError("You may only deselect options of a multi-select")
+            raise NotImplementedError("只能对多项选框执行反选。")
 
         for i in self.options:
             i.click()
@@ -845,9 +862,9 @@ def _wait_ele(page_or_ele,
     :return: 等待是否成功
     """
     if mode.lower() not in ('del', 'display', 'hidden'):
-        raise ValueError('Argument mode can only be "del", "display", "hidden"')
+        raise ValueError('mode参数只能是"del"、"display"或"hidden"。')
 
-    if isinstance(page_or_ele, DrissionElement):  # TODO: 是否要改为 BaseElement
+    if isinstance(page_or_ele, BaseElement):
         page = page_or_ele.page
         ele_or_driver = page_or_ele.inner_ele
     else:
@@ -871,7 +888,7 @@ def _wait_ele(page_or_ele,
         pass
 
     else:
-        raise TypeError('The type of loc_or_ele can only be str, tuple, DriverElement, WebElement')
+        raise TypeError('loc_or_ele参数只能是str、tuple、DriverElement 或 WebElement类型')
 
     # 当传入参数是元素对象时
     if is_ele:
