@@ -4,15 +4,15 @@
 @Contact :   g1879@qq.com
 @File    :   session_element.py
 """
-from re import match, DOTALL, sub
+from re import match, DOTALL
 from typing import Union, List, Tuple
 from urllib.parse import urlparse, urljoin, urlunparse
 
 from lxml.etree import tostring
 from lxml.html import HtmlElement, fromstring
 
-from .base import DrissionElement
-from .common import str_to_loc, translate_loc, format_html
+from .base import DrissionElement, BasePage, BaseElement
+from .common import str_to_loc, translate_loc, get_ele_txt
 
 
 class SessionElement(DrissionElement):
@@ -25,15 +25,13 @@ class SessionElement(DrissionElement):
         attrs = [f"{attr}='{self.attrs[attr]}'" for attr in self.attrs]
         return f'<SessionElement {self.tag} {" ".join(attrs)}>'
 
-    def __call__(self, loc_or_str: Union[Tuple[str, str], str], mode: str = 'single', timeout: float = None):
+    def __call__(self, loc_or_str: Union[Tuple[str, str], str]):
         """在内部查找元素                                            \n
-        例：ele2 = ele1('@id=ele_id')                               \n
+        例：ele2 = ele1('@id=ele_id')                              \n
         :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
-        :param mode: 'single' 或 'all'，对应查找一个或全部
-        :param timeout: 不起实际作用，用于和父类对应
-        :return: SessionElement对象
+        :return: SessionElement对象或属性、文本
         """
-        return super().__call__(loc_or_str, mode, timeout)
+        return self.ele(loc_or_str)
 
     @property
     def tag(self) -> str:
@@ -42,8 +40,8 @@ class SessionElement(DrissionElement):
 
     @property
     def html(self) -> str:
-        """返回元素outerHTML文本"""
-        html = format_html(tostring(self._inner_ele, method="html").decode())
+        """返回outerHTML文本"""
+        html = tostring(self._inner_ele, method="html").decode()
         return html[:html.rfind('>') + 1]  # tostring()会把跟紧元素的文本节点也带上，因此要去掉
 
     @property
@@ -60,45 +58,18 @@ class SessionElement(DrissionElement):
     @property
     def text(self) -> str:
         """返回元素内所有文本"""
-
-        # 为尽量保证与浏览器结果一致，弄得比较复杂
-        def get_node(ele, pre: bool = False):
-            str_list = []
-            if ele.tag == 'pre':
-                pre = True
-
-            current_tag = None
-            for el in ele.eles('xpath:./text() | *'):
-                if current_tag in ('br', 'p') and str_list and str_list[-1] != '\n':
-                    str_list.append('\n')
-
-                if isinstance(el, str):
-                    if sub('[ \n]', '', el) != '':
-                        if pre:
-                            str_list.append(el)
-                        else:
-                            str_list.append(el.replace('\n', ' ').strip(' \t'))
-
-                    elif '\n' in el and str_list and str_list[-1] != '\n':
-                        str_list.append('\n')
-                    else:
-                        str_list.append(' ')
-                    current_tag = None
-                else:
-                    str_list.extend(get_node(el, pre))
-                    current_tag = el.tag
-
-            return str_list
-
-        re_str = ''.join(get_node(self))
-        re_str = sub(r' {2,}', ' ', re_str)
-        return format_html(re_str, False)
+        return get_ele_txt(self)
 
     @property
     def raw_text(self) -> str:
         """返回未格式化处理的元素内文本"""
         return str(self._inner_ele.text_content())
 
+    @property
+    def parent(self):
+        """返回父级元素"""
+        return self.parents()
+
     def parents(self, num: int = 1):
         """返回上面第num级父元素                                         \n
         :param num: 第几级父元素
@@ -114,22 +85,23 @@ class SessionElement(DrissionElement):
         # 获取href属性时返回绝对url
         if attr == 'href':
             link = self.inner_ele.get('href')
-
             # 若为链接为None、js或邮件，直接返回
             if not link or link.lower().startswith(('javascript:', 'mailto:')):
                 return link
 
-            # 其它情况直接返回绝对url
-            else:
+            else:  # 其它情况直接返回绝对url
                 return self._make_absolute(link)
 
         elif attr == 'src':
             return self._make_absolute(self.inner_ele.get('src'))
 
-        elif attr in ('text', 'innerText'):
+        elif attr == 'text':
             return self.text
 
-        elif attr == 'outerHTML':
+        elif attr == 'innerText':
+            return self.raw_text
+
+        elif attr in ('html', 'outerHTML'):
             return self.html
 
         elif attr == 'innerHTML':
@@ -138,45 +110,44 @@ class SessionElement(DrissionElement):
         else:
             return self.inner_ele.get(attr)
 
-    def ele(self, loc_or_str: Union[Tuple[str, str], str], mode: str = None, timeout=None):
-        """返回当前元素下级符合条件的子元素、属性或节点文本，默认返回第一个                                      \n
+    def ele(self, loc_or_str: Union[Tuple[str, str], str], timeout=None):
+        """返回当前元素下级符合条件的第一个元素、属性或节点文本                      \n
         :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
-        :param mode: 'single' 或 'all‘，对应查找一个或全部
-        :param timeout: 不起实际作用，用于和父类对应
-        :return: SessionElement对象
+        :param timeout: 不起实际作用，用于和DriverElement对应，便于无差别调用
+        :return: SessionElement对象或属性、文本
         """
-        if isinstance(loc_or_str, (str, tuple)):
-            if isinstance(loc_or_str, str):
-                loc_or_str = str_to_loc(loc_or_str)
-            else:
-                if len(loc_or_str) != 2:
-                    raise ValueError("Len of loc_or_str must be 2 when it's a tuple.")
-                loc_or_str = translate_loc(loc_or_str)
-        else:
-            raise ValueError('Argument loc_or_str can only be tuple or str.')
-
-        element = self
-        loc_str = loc_or_str[1]
-
-        if loc_or_str[0] == 'xpath' and loc_or_str[1].lstrip().startswith('/'):
-            loc_str = f'.{loc_str}'
-
-        # 若css以>开头，表示找元素的直接子元素，要用page以绝对路径才能找到
-        if loc_or_str[0] == 'css selector' and loc_or_str[1].lstrip().startswith('>'):
-            loc_str = f'{self.css_path}{loc_or_str[1]}'
-            element = self.page
-
-        loc_or_str = loc_or_str[0], loc_str
-
-        return execute_session_find(element, loc_or_str, mode)
+        return self._ele(loc_or_str)
 
     def eles(self, loc_or_str: Union[Tuple[str, str], str], timeout=None):
-        """返回当前元素下级所有符合条件的子元素、属性或节点文本                                                   \n
+        """返回当前元素下级所有符合条件的子元素、属性或节点文本                       \n
+        :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
+        :param timeout: 不起实际作用，用于和DriverElement对应，便于无差别调用
+        :return: SessionElement对象或属性、文本组成的列表
+        """
+        return self._ele(loc_or_str, single=False)
+
+    def s_ele(self, loc_or_str: Union[Tuple[str, str], str] = None):
+        """返回当前元素下级符合条件的第一个元素、属性或节点文本                       \n
+        :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
+        :return: SessionElement对象或属性、文本
+        """
+        return self._ele(loc_or_str)
+
+    def s_eles(self, loc_or_str: Union[Tuple[str, str], str]):
+        """返回当前元素下级所有符合条件的子元素、属性或节点文本                       \n
+        :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
+        :return: SessionElement对象或属性、文本组成的列表
+        """
+        return self._ele(loc_or_str, single=False)
+
+    def _ele(self, loc_or_str: Union[Tuple[str, str], str], timeout=None, single: bool = True):
+        """返回当前元素下级符合条件的子元素、属性或节点文本，默认返回第一个           \n
         :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
         :param timeout: 不起实际作用，用于和父类对应
-        :return: SessionElement对象组成的列表
+        :param single: True则返回第一个，False则返回全部
+        :return: SessionElement对象
         """
-        return self.ele(loc_or_str, mode='all')
+        return make_session_ele(self, loc_or_str, single)
 
     def _get_ele_path(self, mode) -> str:
         """获取css路径或xpath路径
@@ -223,45 +194,91 @@ class SessionElement(DrissionElement):
         return link
 
 
-def execute_session_find(page_or_ele,
-                         loc: Tuple[str, str],
-                         mode: str = 'single', ) -> Union[SessionElement, List[SessionElement], str, None]:
-    """执行session模式元素的查找                              \n
-    页面查找元素及元素查找下级元素皆使用此方法                   \n
-    :param page_or_ele: SessionPage对象或SessionElement对象
-    :param loc: 元素定位元组
-    :param mode: 'single' 或 'all'，对应获取第一个或全部
-    :return: 返回SessionElement元素或列表
+def make_session_ele(html_or_ele: Union[str, BaseElement, BasePage],
+                     loc: Union[str, Tuple[str, str]] = None,
+                     single: bool = True) -> Union[SessionElement, List[SessionElement], str, None]:
+    """从接收到的对象或html文本中查找元素，返回SessionElement对象                 \n
+    如要直接从html生成SessionElement而不在下级查找，loc输入None即可               \n
+    :param html_or_ele: html文本、BaseParser对象
+    :param loc: 定位元组或字符串，为None时不在下级查找，返回根元素
+    :param single: True则返回第一个，False则返回全部
+    :return: 返回SessionElement元素或列表，或属性文本
     """
-    mode = mode or 'single'
-    if mode not in ('single', 'all'):
-        raise ValueError(f"Argument mode can only be 'single' or 'all', not '{mode}'.")
+    # ---------------处理定位符---------------
+    if not loc:
+        loc = ('xpath', '.')
+        single = True
+    elif isinstance(loc, str):
+        loc = str_to_loc(loc)
+    elif isinstance(loc, tuple):
+        loc = translate_loc(loc)
+    else:
+        raise ValueError("定位符必须为str或长度为2的tuple。")
 
-    # 根据传入对象类型获取页面对象和lxml元素对象
-    if isinstance(page_or_ele, SessionElement):
-        page = page_or_ele.page
-        page_or_ele = page_or_ele.inner_ele
-    else:  # 传入的是SessionPage对象
-        page = page_or_ele
-        page_or_ele = fromstring(sub(r'&nbsp;?', '&nbsp;', page_or_ele.response.text))
+    # ---------------根据传入对象类型获取页面对象和lxml元素对象---------------
+    if isinstance(html_or_ele, SessionElement):  # SessionElement
+        page = html_or_ele.page
 
-    try:
-        # 用lxml内置方法获取lxml的元素对象列表
-        if loc[0] == 'xpath':
-            ele = page_or_ele.xpath(loc[1])
+        loc_str = loc[1]
+        if loc[0] == 'xpath' and loc[1].lstrip().startswith('/'):
+            loc_str = f'.{loc[1]}'
+            html_or_ele = html_or_ele.inner_ele
+
+        # 若css以>开头，表示找元素的直接子元素，要用page以绝对路径才能找到
+        elif loc[0] == 'css selector' and loc[1].lstrip().startswith('>'):
+            loc_str = f'{html_or_ele.css_path}{loc[1]}'
+            if html_or_ele.page:
+                html_or_ele = fromstring(html_or_ele.page.html)
+            else:  # 接收html文本，无page的情况
+                html_or_ele = fromstring(html_or_ele('xpath:/ancestor::*').html)
 
-        # 用css selector获取元素对象列表
         else:
-            ele = page_or_ele.cssselect(loc[1])
+            html_or_ele = html_or_ele.inner_ele
 
-        # 结果不是列表，如数字
-        if not isinstance(ele, list):
+        loc = loc[0], loc_str
+
+    elif isinstance(html_or_ele, DrissionElement):  # DriverElement
+        loc_str = loc[1]
+        if loc[0] == 'xpath' and loc[1].lstrip().startswith('/'):
+            loc_str = f'.{loc[1]}'
+        elif loc[0] == 'css selector' and loc[1].lstrip().startswith('>'):
+            loc_str = f'{html_or_ele.css_path}{loc[1]}'
+        loc = loc[0], loc_str
+
+        # 获取整个页面html再定位到当前元素，以实现查找上级元素
+        page = html_or_ele.page
+        xpath = html_or_ele.xpath
+        html_or_ele = fromstring(html_or_ele.page.html)
+        html_or_ele = html_or_ele.xpath(xpath)[0]
+
+    elif isinstance(html_or_ele, BasePage):  # MixPage, DriverPage 或 SessionPage
+        page = html_or_ele
+        html_or_ele = fromstring(html_or_ele.html)
+
+    elif isinstance(html_or_ele, str):  # 直接传入html文本
+        page = None
+        html_or_ele = fromstring(html_or_ele)
+
+    elif isinstance(html_or_ele, BaseElement):  # ShadowRootElement
+        page = html_or_ele.page
+        html_or_ele = fromstring(html_or_ele.html)
+
+    else:
+        raise TypeError('html_or_ele参数只能是元素、页面对象或html文本。')
+
+    # ---------------执行查找-----------------
+    try:
+        if loc[0] == 'xpath':  # 用lxml内置方法获取lxml的元素对象列表
+            ele = html_or_ele.xpath(loc[1])
+        else:  # 用css selector获取元素对象列表
+            ele = html_or_ele.cssselect(loc[1])
+
+        if not isinstance(ele, list):  # 结果不是列表，如数字
             return ele
 
         # 把lxml元素对象包装成SessionElement对象并按需要返回第一个或全部
-        if mode == 'single':
+        if single:
             ele = ele[0] if ele else None
-
             if isinstance(ele, HtmlElement):
                 return SessionElement(ele, page)
             elif isinstance(ele, str):
@@ -269,14 +286,13 @@ def execute_session_find(page_or_ele,
             else:
                 return None
 
-        elif mode == 'all':
+        else:  # 返回全部
             return [SessionElement(e, page) if isinstance(e, HtmlElement) else e for e in ele if e != '\n']
 
     except Exception as e:
-
         if 'Invalid expression' in str(e):
-            raise SyntaxError(f'Invalid xpath syntax. {loc}')
+            raise SyntaxError(f'无效的xpath语句：{loc}')
         elif 'Expected selector' in str(e):
-            raise SyntaxError(f'Invalid css selector syntax. {loc}')
+            raise SyntaxError(f'无效的css select语句：{loc}')
 
         raise e
