@@ -7,7 +7,7 @@
 from os import path as os_PATH, sep
 from pathlib import Path
 from random import randint
-from re import search as re_SEARCH, sub
+from re import search, sub
 from time import time, sleep
 from typing import Union, List, Tuple
 from urllib.parse import urlparse, quote, unquote
@@ -16,9 +16,9 @@ from requests import Session, Response
 from tldextract import extract
 
 from .base import BasePage
-from .common import str_to_loc, translate_loc, get_available_file_name, format_html
+from .common import get_usable_path, make_valid_name
 from .config import _cookie_to_dict
-from .session_element import SessionElement, execute_session_find
+from .session_element import SessionElement, make_session_ele
 
 
 class SessionPage(BasePage):
@@ -30,18 +30,14 @@ class SessionPage(BasePage):
         self._session = session
         self._response = None
 
-    def __call__(self,
-                 loc_or_str: Union[Tuple[str, str], str, SessionElement],
-                 mode: str = 'single',
-                 timeout: float = None) -> Union[SessionElement, List[SessionElement]]:
+    def __call__(self, loc_or_str: Union[Tuple[str, str], str, SessionElement]) \
+            -> Union[SessionElement, List[SessionElement], str]:
         """在内部查找元素                                            \n
-        例：ele2 = ele1('@id=ele_id')                               \n
+        例：ele2 = ele1('@id=ele_id')                              \n
         :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
-        :param mode: 'single' 或 'all'，对应查找一个或全部
-        :param timeout: 不起实际作用，用于和父类对应
-        :return: SessionElement对象
+        :return: SessionElement对象或属性文本
         """
-        return super().__call__(loc_or_str, mode, timeout)
+        return self.ele(loc_or_str)
 
     # -----------------共有属性和方法-------------------
     @property
@@ -51,8 +47,8 @@ class SessionPage(BasePage):
 
     @property
     def html(self) -> str:
-        """返回页面html文本"""
-        return format_html(self.response.text) if self.response else ''
+        """返回页面的html文本"""
+        return self.response.text if self.response else ''
 
     @property
     def json(self) -> dict:
@@ -94,48 +90,54 @@ class SessionPage(BasePage):
 
             else:
                 if show_errmsg:
-                    raise ConnectionError(f'{to_url}\nStatus code: {self._response.status_code}.')
+                    raise ConnectionError(f'{to_url}\n连接状态码：{self._response.status_code}.')
 
                 self._url_available = False
 
         return self._url_available
 
-    def ele(self,
-            loc_or_ele: Union[Tuple[str, str], str, SessionElement],
-            mode: str = None, timeout=None) -> Union[SessionElement, List[SessionElement], str, None]:
+    def ele(self, loc_or_ele: Union[Tuple[str, str], str, SessionElement], timeout=None) \
+            -> Union[SessionElement, List[SessionElement], str, None]:
+        """返回页面中符合条件的第一个元素、属性或节点文本                            \n
+        :param loc_or_ele: 元素的定位信息，可以是元素对象，loc元组，或查询字符串
+        :param timeout: 不起实际作用，用于和DriverElement对应，便于无差别调用
+        :return: SessionElement对象或属性、文本
+        """
+        return self._ele(loc_or_ele)
+
+    def eles(self, loc_or_str: Union[Tuple[str, str], str], timeout=None) -> List[SessionElement]:
+        """返回页面中所有符合条件的元素、属性或节点文本                          \n
+        :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
+        :param timeout: 不起实际作用，用于和DriverElement对应，便于无差别调用
+        :return: SessionElement对象或属性、文本组成的列表
+        """
+        return self._ele(loc_or_str, single=False)
+
+    def s_ele(self, loc_or_str: Union[Tuple[str, str], str]):
+        """返回页面中符合条件的第一个元素、属性或节点文本                          \n
+        :param loc_or_str: 元素的定位信息，可以是元素对象，loc元组，或查询字符串
+        :return: SessionElement对象或属性、文本
+        """
+        return self._ele(loc_or_str)
+
+    def s_eles(self, loc_or_str: Union[Tuple[str, str], str]):
+        """返回页面中符合条件的所有元素、属性或节点文本                              \n
+        :param loc_or_str: 元素的定位信息，可以是元素对象，loc元组，或查询字符串
+        :return: SessionElement对象或属性、文本
+        """
+        return self._ele(loc_or_str, single=False)
+
+    def _ele(self,
+             loc_or_ele: Union[Tuple[str, str], str, SessionElement],
+             timeout: float = None,
+             single: bool = True) -> Union[SessionElement, List[SessionElement], str, None]:
         """返回页面中符合条件的元素、属性或节点文本，默认返回第一个                                           \n
         :param loc_or_ele: 元素的定位信息，可以是元素对象，loc元组，或查询字符串
-        :param mode: 'single' 或 'all‘，对应查找一个或全部
         :param timeout: 不起实际作用，用于和父类对应
+        :param single: True则返回第一个，False则返回全部
         :return: SessionElement对象
         """
-        if isinstance(loc_or_ele, (str, tuple)):
-            if isinstance(loc_or_ele, str):
-                loc_or_ele = str_to_loc(loc_or_ele)
-            else:
-                if len(loc_or_ele) != 2:
-                    raise ValueError("Len of loc_or_ele must be 2 when it's a tuple.")
-                loc_or_ele = translate_loc(loc_or_ele)
-
-        elif isinstance(loc_or_ele, SessionElement):
-            return loc_or_ele
-
-        else:
-            raise ValueError('Argument loc_or_str can only be tuple, str, SessionElement, Element.')
-
-        return execute_session_find(self, loc_or_ele, mode)
-
-    def eles(self,
-             loc_or_str: Union[Tuple[str, str], str], timeout=None) -> List[SessionElement]:
-        """返回页面中所有符合条件的元素、属性或节点文本                                                     \n
-        :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
-        :param timeout: 不起实际作用，用于和父类对应
-        :return: SessionElement对象组成的列表
-        """
-        if not isinstance(loc_or_str, (tuple, str)):
-            raise TypeError('Type of loc_or_str can only be tuple or str.')
-
-        return super().eles(loc_or_str, timeout)
+        return loc_or_ele if isinstance(loc_or_ele, SessionElement) else make_session_ele(self, loc_or_ele, single)
 
     def get_cookies(self, as_dict: bool = False, all_domains: bool = False) -> Union[dict, list]:
         """返回cookies                               \n
@@ -163,7 +165,7 @@ class SessionPage(BasePage):
                         times: int = 0,
                         interval: float = 1,
                         mode: str = 'get',
-                        data: dict = None,
+                        data: Union[dict, str] = None,
                         show_errmsg: bool = False,
                         **kwargs) -> Response:
         """尝试连接，重试若干次                            \n
@@ -194,7 +196,7 @@ class SessionPage(BasePage):
                 print(f'重试 {to_url}')
 
         if not r and show_errmsg:
-            raise err if err is not None else ConnectionError('Connect error.')
+            raise err if err is not None else ConnectionError('连接异常。')
 
         return r
 
@@ -211,7 +213,7 @@ class SessionPage(BasePage):
 
     def post(self,
              url: str,
-             data: dict = None,
+             data: Union[dict, str] = None,
              go_anyway: bool = True,
              show_errmsg: bool = False,
              retry: int = None,
@@ -246,7 +248,7 @@ class SessionPage(BasePage):
 
             else:
                 if show_errmsg:
-                    raise ConnectionError(f'Status code: {self._response.status_code}.')
+                    raise ConnectionError(f'连接状态码：{self._response.status_code}.')
                 self._url_available = False
 
         return self._url_available
@@ -256,7 +258,7 @@ class SessionPage(BasePage):
                  goal_path: str,
                  rename: str = None,
                  file_exists: str = 'rename',
-                 post_data: dict = None,
+                 post_data: Union[str, dict] = None,
                  show_msg: bool = False,
                  show_errmsg: bool = False,
                  retry: int = None,
@@ -267,128 +269,87 @@ class SessionPage(BasePage):
         :param goal_path: 存放路径
         :param rename: 重命名文件，可不写扩展名
         :param file_exists: 若存在同名文件，可选择 'rename', 'overwrite', 'skip' 方式处理
-        :param post_data: post方式的数据
+        :param post_data: post方式的数据，这个参数不为None时自动转成post方式
         :param show_msg: 是否显示下载信息
         :param show_errmsg: 是否抛出和显示异常
         :param retry: 重试次数
         :param interval: 重试间隔时间
         :param kwargs: 连接参数
-        :return: 下载是否成功（bool）和状态信息（成功时信息为文件路径）的元组
+        :return: 下载是否成功（bool）和状态信息（成功时信息为文件路径）的元组，跳过时第一位为None
         """
         if file_exists == 'skip' and Path(f'{goal_path}{sep}{rename}').exists():
             if show_msg:
-                print(f'{file_url}\n{goal_path}{sep}{rename}\nSkipped.\n')
+                print(f'{file_url}\n{goal_path}{sep}{rename}\n存在同名文件，已跳过。\n')
+            return None, '已跳过，因存在同名文件。'
 
-            return False, 'Skipped because a file with the same name already exists.'
+        def do() -> tuple:
+            kwargs['stream'] = True
+            if 'timeout' not in kwargs:
+                kwargs['timeout'] = 20
 
-        def do(url: str,
-               goal: str,
-               new_name: str = None,
-               exists: str = 'rename',
-               data: dict = None,
-               msg: bool = False,
-               errmsg: bool = False,
-               **args) -> tuple:
-            args['stream'] = True
-
-            if 'timeout' not in args:
-                args['timeout'] = 20
-
-            mode = 'post' if data else 'get'
-            # 生成的response不写入self._response，是临时的
-            r, info = self._make_response(url, mode=mode, data=data, show_errmsg=errmsg, **args)
+            # 生成临时的response
+            mode = 'post' if post_data is not None else 'get'
+            r, info = self._make_response(file_url, mode=mode, data=post_data, show_errmsg=show_errmsg, **kwargs)
 
             if r is None:
-                if msg:
+                if show_msg:
                     print(info)
-
                 return False, info
 
             if not r.ok:
-                if errmsg:
-                    raise ConnectionError(f'Status code: {r.status_code}.')
-
-                return False, f'Status code: {r.status_code}.'
+                if show_errmsg:
+                    raise ConnectionError(f'连接状态码：{r.status_code}')
+                return False, f'状态码：{r.status_code}'
 
             # -------------------获取文件名-------------------
-            file_name = ''
-            content_disposition = r.headers.get('content-disposition')
-
-            # 使用header里的文件名
-            if content_disposition:
-                file_name = content_disposition.encode('ISO-8859-1').decode('utf-8')
-                file_name = re_SEARCH(r'filename *= *"?([^";]+)', file_name)
-
-                if file_name:
-                    file_name = file_name.group(1)
-
-                    if file_name[0] == file_name[-1] == "'":
-                        file_name = file_name[1:-1]
-
-            # 在url里获取文件名
-            if not file_name and os_PATH.basename(url):
-                file_name = os_PATH.basename(url).split("?")[0]
-
-            # 找不到则用时间和随机数生成文件名
-            if not file_name:
-                file_name = f'untitled_{time()}_{randint(0, 100)}'
-
-            # 去除非法字符
-            file_name = sub(r'[\\/*:|<>?"]', '', file_name).strip()
-            file_name = unquote(file_name)
+            file_name = _get_download_file_name(file_url, r)
 
             # -------------------重命名，不改变扩展名-------------------
-            if new_name:
-                new_name = sub(r'[\\/*:|<>?"]', '', new_name).strip()
+            if rename:
                 ext_name = file_name.split('.')[-1]
-
-                if '.' in new_name or ext_name == file_name:
-                    full_name = new_name
+                if '.' in rename or ext_name == file_name:  # 新文件名带后缀或原文件名没有后缀
+                    full_name = rename
                 else:
-                    full_name = f'{new_name}.{ext_name}'
-
+                    full_name = f'{rename}.{ext_name}'
             else:
                 full_name = file_name
 
+            full_name = make_valid_name(full_name)
+
             # -------------------生成路径-------------------
-            goal_Path = Path(goal)
-            goal = ''
+            goal_Path = Path(goal_path)
             skip = False
 
-            for key, p in enumerate(goal_Path.parts):  # 去除路径中的非法字符
-                goal += goal_Path.drive if key == 0 and goal_Path.drive else sub(r'[*:|<>?"]', '', p).strip()
-                goal += '\\' if p != '\\' and key < len(goal_Path.parts) - 1 else ''
-
-            goal_Path = Path(goal).absolute()
-            goal_Path.mkdir(parents=True, exist_ok=True)
+            # 按windows规则去除路径中的非法字符
+            goal = goal_Path.anchor + sub(r'[*:|<>?"]', '', goal_path.lstrip(goal_Path.anchor)).strip()
+            Path(goal).absolute().mkdir(parents=True, exist_ok=True)
             full_path = Path(f'{goal}{sep}{full_name}')
 
             if full_path.exists():
                 if file_exists == 'rename':
-                    full_name = get_available_file_name(goal, full_name)
-                    full_path = Path(f'{goal}{sep}{full_name}')
+                    full_path = get_usable_path(f'{goal}{sep}{full_name}')
+                    full_name = full_path.name
 
-                elif exists == 'skip':
+                elif file_exists == 'skip':
                     skip = True
 
-                elif exists == 'overwrite':
+                elif file_exists == 'overwrite':
                     pass
 
                 else:
-                    raise ValueError("Argument file_exists can only be 'skip', 'overwrite', 'rename'.")
+                    raise ValueError("file_exists参数只能是'skip'、'overwrite' 或 'rename'。")
 
             # -------------------打印要下载的文件-------------------
-            if msg:
+            if show_msg:
                 print(file_url)
                 print(full_name if file_name == full_name else f'{file_name} -> {full_name}')
-                print(f'Downloading to: {goal}')
-
+                print(f'正在下载到：{goal}')
                 if skip:
-                    print('Skipped.\n')
+                    print('存在同名文件，已跳过。\n')
 
             # -------------------开始下载-------------------
             if skip:
-                return False, 'Skipped because a file with the same name already exists.'
+                return None, '已跳过，因存在同名文件。'
 
             # 获取远程文件大小
             content_length = r.headers.get('content-length')
@@ -404,36 +365,32 @@ class SessionPage(BasePage):
                             tmpFile.write(chunk)
 
                             # 如表头有返回文件大小，显示进度
-                            if msg and file_size:
+                            if show_msg and file_size:
                                 downloaded_size += 1024
                                 rate = downloaded_size / file_size if downloaded_size < file_size else 1
                                 print('\r {:.0%} '.format(rate), end="")
 
             except Exception as e:
-                if errmsg:
+                if show_errmsg:
                     raise ConnectionError(e)
-
-                download_status, info = False, f'Download failed.\n{e}'
+                download_status, info = False, f'下载失败。\n{e}'
 
             else:
                 if full_path.stat().st_size == 0:
-                    if errmsg:
-                        raise ValueError('File size is 0.')
-
-                    download_status, info = False, 'File size is 0.'
+                    if show_errmsg:
+                        raise ValueError('文件大小为0。')
+                    download_status, info = False, '文件大小为0。'
 
                 else:
                     download_status, info = True, str(full_path)
 
             finally:
-                # 删除下载出错文件
-                if not download_status and full_path.exists():
-                    full_path.unlink()
-
+                if download_status is False and full_path.exists():
+                    full_path.unlink()  # 删除下载出错文件
                 r.close()
 
             # -------------------显示并返回值-------------------
-            if msg:
+            if show_msg:
                 print(info, '\n')
 
             info = f'{goal}{sep}{full_name}' if download_status else info
@@ -441,15 +398,16 @@ class SessionPage(BasePage):
 
         retry_times = retry or self.retry_times
         retry_interval = interval or self.retry_interval
-        result = do(file_url, goal_path, rename, file_exists, post_data, show_msg, show_errmsg, **kwargs)
+        result = do()
 
-        if not result[0] and not str(result[1]).startswith('Skipped'):
+        if result[0] is False:  # 第一位为None表示跳过的情况
             for i in range(retry_times):
                 sleep(retry_interval)
+                if show_msg:
+                    print(f'\n重试 {file_url}')
 
-                print(f'重试 {file_url}')
-                result = do(file_url, goal_path, rename, file_exists, post_data, show_msg, show_errmsg, **kwargs)
-                if result[0]:
+                result = do()
+                if result[0] is not False:
                     break
 
         return result
@@ -457,7 +415,7 @@ class SessionPage(BasePage):
     def _make_response(self,
                        url: str,
                        mode: str = 'get',
-                       data: dict = None,
+                       data: Union[dict, str] = None,
                        show_errmsg: bool = False,
                        **kwargs) -> tuple:
         """生成response对象                     \n
@@ -470,11 +428,11 @@ class SessionPage(BasePage):
         """
         if not url:
             if show_errmsg:
-                raise ValueError('url is empty.')
-            return None, 'url is empty.'
+                raise ValueError('URL为空。')
+            return None, 'URL为空。'
 
         if mode not in ('get', 'post'):
-            raise ValueError("Argument mode can only be 'get' or 'post'.")
+            raise ValueError("mode参数只能是'get'或'post'。")
 
         url = quote(url, safe='/:&?=%;#@+!')
 
@@ -518,14 +476,14 @@ class SessionPage(BasePage):
             # ----------------获取并设置编码开始-----------------
             # 在headers中获取编码
             content_type = r.headers.get('content-type', '').lower()
-            charset = re_SEARCH(r'charset[=: ]*(.*)?[;]', content_type)
+            charset = search(r'charset[=: ]*(.*)?[;]', content_type)
 
             if charset:
                 r.encoding = charset.group(1)
 
             # 在headers中获取不到编码，且如果是网页
             elif content_type.replace(' ', '').startswith('text/html'):
-                re_result = re_SEARCH(b'<meta.*?charset=[ \\\'"]*([^"\\\' />]+).*?>', r.content)
+                re_result = search(b'<meta.*?charset=[ \\\'"]*([^"\\\' />]+).*?>', r.content)
 
                 if re_result:
                     charset = re_result.group(1).decode()
@@ -536,3 +494,46 @@ class SessionPage(BasePage):
             # ----------------获取并设置编码结束-----------------
 
             return r, 'Success'
+
+
+def _get_download_file_name(url, response) -> str:
+    """从headers或url中获取文件名，如果获取不到，生成一个随机文件名
+    :param url: 文件url
+    :param response: 返回的response
+    :return: 下载文件的文件名
+    """
+    file_name = ''
+    charset = ''
+    content_disposition = response.headers.get('content-disposition', '').replace(' ', '')
+
+    # 使用header里的文件名
+    if content_disposition:
+        txt = search(r'filename\*="?([^";]+)', content_disposition)
+        if txt:  # 文件名自带编码方式
+            txt = txt.group(1).split("''", 1)
+            if len(txt) == 2:
+                charset, file_name = txt
+            else:
+                file_name = txt[0]
+
+        else:  # 文件名没带编码方式
+            txt = search(r'filename="?([^";]+)', content_disposition)
+            if txt:
+                file_name = txt.group(1)
+
+                # 获取编码（如有）
+                charset = response.encoding
+
+        file_name = file_name.strip("'")
+
+    # 在url里获取文件名
+    if not file_name and os_PATH.basename(url):
+        file_name = os_PATH.basename(url).split("?")[0]
+
+    # 找不到则用时间和随机数生成文件名
+    if not file_name:
+        file_name = f'untitled_{time()}_{randint(0, 100)}'
+
+    # 去除非法字符
+    charset = charset or 'utf-8'
+    return unquote(file_name, charset)
